import os, glob
from distutils.command import build_py
from Tools import msgfmt

class BuildL10n(build_py.build_py):

    command_name = 'build_l10n'

    def get_source_files(self):
        filenames = []
        for loc in self.l10n:
            src_dir = self.get_package_dir(loc.package)
            po_files = glob.glob(os.path.join(src_dir, '*.po'))
            filenames.extend(po_files)
        return filenames

    def initialize_options(self):
        build_py.build_py.initialize_options(self)
        self.build_temp = None
        self.l10n = None
        self.outfiles = []

    def finalize_options(self):
        build_py.build_py.finalize_options(self)
        self.set_undefined_options('build',
                                   ('build_temp', 'build_temp'))
        self.l10n = self.distribution.l10n or []

    def run(self):
        for loc in self.l10n:
            package_path = loc.package.split('.')
            package_path = apply(os.path.join, package_path)

            lib_dir = os.path.join(self.build_lib, package_path)

            # Make sure the generate directory exists
            temp_dir = os.path.join(self.build_temp, package_path)
            self.mkpath(temp_dir)
            
            src_dir = self.get_package_dir(loc.package)
            po_files = glob.glob(os.path.join(src_dir, '*.po'))
            for po_file in po_files:
                (po_filename, copied) = self.copy_file(po_file, temp_dir,
                                                       preserve_mode=0)
                if not (copied or self.force):
                    # copy_file() already announced the skip
                    continue

                # Create the locale directory for this .mo file
                locale = os.path.splitext(os.path.basename(po_file))[0]
                locale_dir = os.path.join(lib_dir, locale, 'LC_MESSAGES')
                self.mkpath(locale_dir)

                # What the msgfmt generated file will eventually be called
                locale_mo = os.path.join(locale_dir, '%s.mo' % loc.domain)
                self.outfiles.append(locale_mo)

                if not self.dry_run:
                    # Only generate the .mo if they user really means it
                    msgfmt.MESSAGES = {}
                    msgfmt.make(po_filename)
                    mo_filename = po_filename[:-2] + 'mo'
                    self.copy_file(mo_filename, locale_mo, preserve_mode=0)

    def get_outputs(self, include_bytecode=1):
        outputs = build_py.build_py.get_outputs(self, include_bytecode)
        outputs.extend(self.outfiles)
        return outputs
