########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Rdf/Parsers/Versa/ResourceExpressions.py,v 1.22 2003/05/27 17:24:31 uogbuji Exp $
"""
Implementation and utilities for resource processing Versa

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

import DataTypes
from Ft.Rdf import RDF_MS_BASE, RDF_SCHEMA_BASE
from Ft.Rdf import OBJECT_TYPE_RESOURCE, OBJECT_TYPE_UNKNOWN
from Ft.Rdf.Drivers import PROPERTIES
from Ft.Xml.XPath import Util

class ResourceExpression:
    """
    Base class for all resource expressions
    """
    pass

class PureQNameExpression(ResourceExpression):
    """
    Expand a QName into a string.  Uses the NsMapping on the context
    """
    def __init__(self,expr):
        self.expr = expr

    def evaluate(self,con):
        res = Util.ExpandQName(self.expr, namespaces=con.nsMapping)
        if res[0] is None:
            return res[1]
        return DataTypes.Resource(res[0] + res[1])


class LiteralExpression(ResourceExpression):
    """
    Use the resource as is
    """
    def __init__(self, value):
        self.value = value

    def evaluate(self, con):
        return self.value


class CurrentExpression(ResourceExpression):
    """
    Return the current resource if there is one
    """
    def evaluate(self,con):
        return con.current


def GetRelations(start, property, con, inverse, useSubProps=1,
                 forceTransitive=0):
    """
    Returns a dictionary of dictionaries.  The outer dict maps properties
    to relations.  The relation dicts match subjects to objects for each
    property.
    """
    #print "GetRelations"
    property = DataTypes.ToResource(property)
    start = start and DataTypes.ToSet(start) or None
    suspects = con.driver.complete(None, str(property), None, None, con.scope, {})
    if useSubProps:
        subprops = con.getCachedSuperProps().get(property, [])
        for sp in subprops:
            suspects.extend(con.driver.complete(None, str(sp), None, None, con.scope, {}))
    sdict = {}
    #print "GetRelations checkpoint 1"
    if not inverse:
        for s in suspects:
            subj = DataTypes.ToResource(s[0])
            obj = s[5] == OBJECT_TYPE_RESOURCE and DataTypes.ToResource(s[2]) or s[2]
            if not sdict.has_key(subj):
                sdict[subj] = []
            sdict[subj].append(obj)
    else:
        for s in suspects:
            subj = DataTypes.ToResource(s[0])
            obj = s[5] == OBJECT_TYPE_RESOURCE and DataTypes.ToResource(s[2]) or s[2]
            if not sdict.has_key(obj):
                sdict[obj] = []
            sdict[obj].append(subj)

    #print "Raw result dict:", sdict
    #Compute transitive closure
    #print "GetRelations checkpoint 2"
    if forceTransitive or con.isTransitive(property):
        #if forceTransitive:
        #    print property, "treated transitively"
        #else:
        #    print property, "is transitive"
        _HandleTransitive(sdict, inverse, con)

    #print "After transitive closure:", sdict
    #Note: in the case of transitives, we have to wait until the end to
    #Cull to start nodes
    #print "GetRelations checkpoint 3"
    if start:
        for k in sdict.keys():
            if not start._d.has_key(k):
                del sdict[k]
    #print "After culling to start nodes:", sdict
    #print "GetRelations end"
    return sdict


def _HandleTransitive(sdict, inverse, con, checked=None, currList=None):
    #FIXME: Possible denial-of-service attack here.  All someone has to do
    #Is create a loop in transitive predicate and infinite loop ahoy
    #print "SDICT:", sdict, checked
    if currList is None: currList = sdict.keys()
    if checked is None: checked = {}
    for subj in currList:
        #print "SUBJ:", subj, subj.__class__
        if checked.has_key(subj):
            continue
        checked[subj] = None
        for obj in sdict[subj]:
            #print "OBJ:", obj, obj.__class__, sdict.has_key(obj), sdict.has_key(DataTypes.ToResource(obj))
            if sdict.has_key(obj):
                if not checked.has_key(obj):
                    checked = _HandleTransitive(sdict, inverse, con, checked, [obj])
                    sdict[subj].extend(sdict[obj])
    return checked

