########################################################################
#
# File Name:            Delete.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import sys, fnmatch, posixpath
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import Schema
from Ft.Rdf import Model

__doc__ = """This command will delete a resource from a 4Suite repository. \
If you have not used the agent or login commands, you will be prompted for \
a username and password of someone who has write access on the resource."""

def Run(options, args):

    try:
        path = posixpath.normpath(args['path'])
        repo = CommandUtil.GetRepository(options, '4ss.delete')

	ta = ""
	if options.has_key('no-traverse'):
	    ta = ';no-traverse'
        commit = 0
        try:
            model = repo.getModel()
            for stmt in model.complete(None, Schema.TYPE, None):
                if fnmatch.fnmatch(stmt.subject, path):
                    print "Deleting: %s" % stmt.subject
                    repo.deleteResource(
                        stmt.subject + ta
                        )
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass

    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('delete',
                          """Delete a resource from the repository""",
                          '/fourthought.com',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('path',
                                                                  "The path of the resource to delete.  This can be a regex expression.",
                                                                  str),
                                       ],
                          options = Options.Options([Options.Option(None,
                                                                    'no-traverse',
                                                                    "Don't traverse aliases"),
                                                     ]),

                          fileName = __file__,
                          )
    return cmd


