########################################################################
#
# File Name:            Login.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """This command will set a user name and password entry into a \
file (on the local filesystem) that can be used for subsequent invocations \
of 4ss command-line scripts. The default location of the file is in the \
current user's home directory. To change the location, set the environment \
variable FTSS_PASSWORD_FILE. This command is an alternative to the agent \
command."""

import sha, getpass

from Ft.Server.Client import Core
from Ft.Server.Client.Core import Util
from Ft.Server.Common import ClAuthenticate

def TextLogin(options, args):

    userName = args.get('username')
    if not userName:
        userName = raw_input("UserName: ")
    passwd = sha.new(getpass.getpass()).hexdigest()

    host = raw_input('Host <localhost>: ')
    host = host or 'localhost'
    port = raw_input('Port <%i>: ' % Core.FTRPC_PORT)
    port = int(port or Core.FTRPC_PORT)

    mang = ClAuthenticate.PasswdManager()
    mang.setUser(userName, passwd, host, port)
    return

def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('login',
                          'Set username and password in a local password file',
                          '',
                          __doc__,
                          function = TextLogin,
                          arguments = [Arguments.OptionalArgument('username',
                                                                  'name of user to log in as.',
                                                                  str),
                                       ],
                          fileName = __file__,
                          )
    return cmd

