########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Remove/Member.py,v 1.5 2003/03/20 03:50:19 mbrown Exp $
"""
Implementation of '4ss remove member' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Removes a User from a Group in the 4Suite repository."""

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

def Run(options,args):

    try:
        commit = 0
        username = args['userName']
        groupname = args['groupName']
        repo = CommandUtil.GetRepository(options,'4ss.remove.member')
        try:
            user = repo.fetchUserOrGroupByName(username)
            group = repo.fetchUserOrGroupByName(groupname)
            if user is not None:
                if group is not None:
                    if user.isResourceType(ResourceTypes.ResourceType.USER):
                        if group.isResourceType(ResourceTypes.ResourceType.GROUP):
                            group.removeMember(user)
                            commit = 1
                        else:
                            print "%s is a user, not a group." % groupname
                    else:
                        print "%s is a group, not a user." % username
                else:
                    print "Group %s does not exist." % groupname
            else:
                print "User %s does not exist." % username
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('member',
                          """Removes a User from a Group in the 4Suite repository.""",
                          'developers michael',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('groupName',
                                                                  "The name of the group.",
                                                                  str),
                                       Arguments.RequiredArgument('userName',
                                                                  "The name of the user.",
                                                                  str),
                                       ],
                          fileName = __file__,
                          )
    return cmd
