########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Set/Owner.py,v 1.4 2003/03/20 03:50:20 mbrown Exp $
"""
Implementation of '4ss set owner' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Modifies the owner of a resource in the 4Suite repository.
The new owner must be an existing User."""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import AclConstants, ResourceTypes


def Run(options, args):
    try:
        commit = 0
        repo = CommandUtil.GetRepository(options, '4ss.set.owner')
        try:
            ownername = args['newowner']
            res = repo.fetchResource(args['name'])
            user = repo.fetchUserOrGroupByName(ownername)
            if user is not None:
                if user.isResourceType(ResourceTypes.ResourceType.USER):
                    res.setOwner(user)
                    commit = 1
                else:
                    sys.stderr.write("%s is a group, not a user.\n" % ownername)
            else:
                sys.stderr.write("User %s does not exist.\n" % ownername)
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('owner',
                          'Modifies the owner of a resource in the 4Suite repository.',
                          '/repo/path/to/doc.xml',
                          __doc__,
                          function=Run,
                          arguments=[Arguments.RequiredArgument('name',
                                                                'name (repo path) of resource whose owner is being changed',
                                                                str),
                                     Arguments.RequiredArgument('newowner',
                                                                'user name of the new owner',
                                                                str),
                                     ],
                          options=Options.Options([
                                                   ]),
                          fileName=__file__,
                          )
    return cmd

