########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Update/RawFile.py,v 1.8 2003/03/20 03:50:20 mbrown Exp $
"""
Implementation of '4ss update rawfile' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Updates the content of an existing Raw File in the 4Suite repository."""

import sys
from Ft.Lib import Uri
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil

def Run(options,args):

    if args['sourceUri'] == '-':
        sourceStream = sys.stdin
    else:
        sourceStream = Uri.BASIC_RESOLVER.resolve(args['sourceUri'])

    text = sourceStream.read()

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.update.rawfile')
        try:
            doc = repo.fetchResource(args['name'])
            doc.setContent(text)
            if options.get('imt'):
                doc.setImt(options['imt'])
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass

    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')

    if args['sourceUri'] != '-':
        sourceStream.close()

def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('rawfile',
                          """Updates the content of an existing Raw File in the 4Suite repository.""",
                          '/images/4suite.jpg /local/path/to/4Suite.jpg',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('name',
                                                                  "The name (repo path) of the file",
                                                                  str),
                                       Arguments.RequiredArgument('sourceUri',
                                                                  "The URI to read the file's content from, or '-' for stdin",
                                                                  str),
                                       ],
                          options = Options.Options([Options.Option(None,
                                                                    'imt=',
                                                                    'The file\'s Internet Media Type. Default: no change.'),
                                                     ]),
                          fileName = __file__,
                          )
    return cmd


