from Ft.Server import FtServerBaseException
import MessageSource
Error = MessageSource.Error

class FtServerClientException(FtServerBaseException):
    MessageSource = MessageSource


import os, sha, getpass

from Ft.Server.Common import ClAuthenticate
from Ft.Server.Client import Core

def GetAuthentication(options=None):
    """
    Returns authentication arguments for connecting to a repository.

    Get Authentication information for a Client connection to the repository
    1: look in the options dictionary for "anonymous"
    2: look in the options dictionary for "username"
    3: look in the options dictionary for "port"
    4: look in the options dictionary for "host"
    5:  If there is no username, then look for the FTSS_AGENT env var to get information
    6: If there still is no username, then look for FTSS_USERNAME env var
    7: If there still is no user name then prompt for it.
    8: look in the options dictionary for "password"
    9: look in the options dictionary for "passwdhash"
    10: If there is no password, then ask the password manager (from 4ss login)
    11: If there still is no password then prompt for it.
    12: If there is no host, then prompt for it
    13: If there is no port then prompt for it
    """
    options = options or {}
    hasUserName = hasPasswd = 0
    userName = None
    passwdHash = None

    # Is this an anonymous login? (ignores other username choices)
    if options.has_key('anonymous'):
        hasUserName = 1
        hasPasswd = 1
    else:
        if options.has_key('username'):
            hasUserName = 1
            userName = options['username']
        if options.has_key('password'):
            hasPasswd = 1
            passwdHash = sha.new(options['password']).hexdigest()
        if options.has_key('passwdhash'):
            hasPasswd = 1
            passwdHash = options['passwdhash']

    host = options.get('host')
    port = int(options.get('port', 0))

    # See if there is an agent (set and non-empty)
    if not hasUserName and os.environ.get('FTSS_AGENT'):
        userName, passwdHash, h, po = eval(os.environ['FTSS_AGENT'])
        hasUserName = hasPasswd = 1
        if not host:
            host = h
        if not port:
            port = int(po)
        
    # See if 'logged in' (set and non-empty)
    if not hasUserName and os.environ.get('FTSS_USERNAME'):
        userName = os.environ['FTSS_USERNAME']
        hasUserName = 1

    #All else failed: prompt user
    if not hasUserName:
        userName = raw_input("4SS user name: ")

    if not hasPasswd:
        pm = ClAuthenticate.PasswdManager()
        pa, h, p = pm.getPassword(userName)
        if pa:
            passwdHash = pa
            hasPasswd = 1
        if not host and h:
            host = h
        if not port and p:
            port = int(p)
        
    if not hasPasswd:
        passwd = getpass.getpass("Password for %s: " % userName)
        passwdHash = sha.new(passwd).hexdigest()

    if not host:
        host = raw_input('Host <%s>: ' % Core.FTRPC_HOST)
        host = host or Core.FTRPC_HOST

    if not port:
        port = raw_input('Port <%i>: ' % Core.FTRPC_PORT)
        port = int(port or Core.FTRPC_PORT)
    
    return (userName, passwdHash, host, port)
    #return {"username": userName, "passwdhash": passwdHash, "host": host, "port": port}


def SmartLogin(options=None):
    """
    Returns a new repository connection.
    """
    userName, passwdHash, host, port = GetAuthentication(options)
    return Core.GetRepository(userName, passwdHash, host, port)

