########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Server/Commands/Log/Clear.py,v 1.6 2003/02/27 09:48:57 mbrown Exp $
"""
4ss_manager log clear command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Clear the logs"""

import os, sys, time

from Ft import SHARE_DIR
from Ft.Lib import Uri
from Ft.Server.Server import FtServerServerException, Error
from Ft.Server.Server.Commands import CommandUtil

def Run(options,args):

    filename = 'error.log'

    try:
        # get repo
        command = '4ss_manager.log.clear'
        user, pwd, props, repo = CommandUtil.GetRepository(options, command)
        localpath = '%s-%s' % (props['CoreId'], filename)
        if type(localpath) == type(u''):
            localpath = localpath.encode('utf-8')
        localpath = os.path.join(SHARE_DIR, 'Data', 'Logs', localpath)

        # clear logfile
        prompt = "Are you sure you want to clear %r? (yes/no)" % filename
        answer = raw_input(prompt)
        if answer.lower() not in ['y', 'yes']:
            return
        when = time.asctime(time.localtime(time.time()))
        log = open(localpath, 'w')
        log.write("Log cleared on %s by %r\n" % (when, user))
        log.close()

        # verify/create the logfile's URI reference in the repo
        try:
            commit = 0
            log = repo.fetchResource('/ftss').fetchResource(filename)

        except FtServerServerException, e:
            if e.code == Error.UNKNOWN_PATH:
                e = FtServerServerException(Error.LOG_URIREF_MISSING, path=e.args['path'])
                prompt = e.message + '\nCreate? (yes/no)'
                answer = raw_input(prompt)
                if answer.lower() not in ['y', 'yes']:
                    return
                src = Uri.OsPathToUri(localpath);
                repopath = '/ftss/' + filename
                imt = 'text/plain'
                doc = repo.createUriReferenceFile(repopath, imt, src)
                print "Created UriReference %r -> %r" % (repopath, src, imt)
                commit = 1
        if commit:
            repo.txCommit()
        else:
            repo.txRollback()

    except FtServerServerException, e:
        sys.stdout.flush()
        sys.stderr.write(e.message)
        if sys.stderr.isatty():
            sys.stderr.write('\n')
        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('clear',
                          'Clear a 4SS Log',
                          '',
                          __doc__,
                          function = Run,
                          fileName = __file__,
                          )
    return cmd


