<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:spec="http://namespaces.fourthought.com/specification"
  xmlns:doc="http://docbook.org/docbook/xml/4.0/namespace"
  version="1.0"
>

  <xsl:import href='docbook_html1.xslt'/>

  <xsl:param name='version'/>

  <xsl:output method='html' indent='yes'/>

  <xsl:param name="FULL_DOCUMENT" select="1"/>

  <xsl:template match="/">
    <xsl:choose>
      <xsl:when test="$FULL_DOCUMENT = 1">
    <HTML>
    <HEAD>
      <TITLE>
        <xsl:value-of select='spec:package/spec:header/spec:name'/>
        <xsl:text>: </xsl:text>
        <xsl:value-of select='spec:package/spec:header/spec:desc'/>
      </TITLE>
      <META NAME="author" CONTENT="{spec:package/spec:header/spec:author}"/>
    </HEAD>
    
    <BODY>
      <xsl:apply-templates/>
    </BODY>
    </HTML>
    </xsl:when>
    <xsl:otherwise>
      <xsl:apply-templates/>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:template>

  <xsl:template match='spec:package'>
    <H1 align='center'><xsl:value-of select='spec:header/spec:name'/></H1>
    <H4 align='center'>
      <xsl:value-of select='spec:header/spec:desc'/>
      <xsl:text> by </xsl:text>
      <A HREF="{spec:header/spec:homepage}"><xsl:value-of select='spec:header/spec:author'/></A>
    </H4>
    <xsl:if test='spec:event'>
      <H3>News</H3>
      <DL>
        <xsl:apply-templates select='spec:event'/>
      </DL>
    </xsl:if>

    <xsl:apply-templates select='spec:overview'/>

    <xsl:apply-templates select='spec:install'/>

    <xsl:if test='spec:structure'>
      <H3>Directory Structure</H3>
      <DL>
        <xsl:apply-templates select='spec:structure'/>
      </DL>
    </xsl:if>
    
    <xsl:if test='spec:note'>
      <H3>Implementation Notes</H3>
      <xsl:apply-templates select='spec:note'/>
    </xsl:if>
    
    <xsl:if test='spec:deviation'>
      <H3>Deviations From The Specification</H3>
      <UL>
        <xsl:apply-templates select='spec:deviation'/>
      </UL>
    </xsl:if>
    
    <xsl:apply-templates select='spec:etc'/>
  </xsl:template>


  <xsl:template match='spec:event'>
    <DT>
      <SAMP><B><xsl:value-of select='spec:date'/></B><xsl:text>:  </xsl:text></SAMP>
      <xsl:value-of select='spec:desc'/>
    </DT>
  </xsl:template>


  <xsl:template match='spec:overview'>
    <H3>Overview</H3>
    <xsl:apply-templates/>
  </xsl:template>
  
  
  <xsl:template match='spec:version'>
  	<xsl:value-of select='$version'/>
  	<xsl:apply-templates/>
  </xsl:template>

  <xsl:template match='spec:install'>
    <H3>Installation</H3>
    <xsl:apply-templates/>
  </xsl:template>
  
  
  <xsl:template match='spec:required'>
  	<H4>Prerequisites</H4>
  	<UL>
  		<xsl:apply-templates/>
  	</UL>
  </xsl:template>

  
  <xsl:template match='spec:structure'>
    <DT><SAMP><xsl:value-of select='spec:path'/></SAMP>
    <xsl:text> - </xsl:text>
    <xsl:value-of select='spec:desc'/></DT>
  </xsl:template>
  

  <xsl:template match='spec:note'>
    <xsl:if test='spec:title'>
      <H4><xsl:value-of select='spec:title'/></H4>
    </xsl:if>
    <xsl:apply-templates/>
  </xsl:template>


  <xsl:template match='spec:deviation'>
    <LI><xsl:apply-templates/></LI>
  </xsl:template>


  <xsl:template match='spec:etc'>
    <H3>More Information</H3>
    <xsl:apply-templates/>
  </xsl:template>


  <!-- ignore any other 'spec:' tags -->
  <xsl:template match="spec:*" priority="-2.0">
    <xsl:apply-templates select="spec:*"/>
  </xsl:template>

</xsl:stylesheet>
