<?xml version="1.0"?>
<!DOCTYPE xsl:stylesheet [
  <!ENTITY nbsp "&#160;">
  <!ENTITY rdf "http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <!ENTITY rdfs "http://www.w3.org/2000/01/rdf-schema#">
  <!ENTITY f	"http://xmlns.4suite.org/ext">
  <!ENTITY fres "http://xmlns.4suite.org/reserved">
  <!ENTITY dc "http://purl.org/dc/elements/1.1/">
  <!ENTITY fschema "http://schemas.4suite.org/4ss">
  <!ENTITY vcard "http://4suite.org/nexus/rdfs/vcard#">
]>
<xsl:stylesheet
  version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:fcore="http://xmlns.4suite.org/4ss/score" 
  xmlns:frdf="http://xmlns.4suite.org/4ss/rdf" 
  xmlns:fhttp="http://xmlns.4suite.org/4ss/http"
  xmlns:exslt="http://exslt.org/common"
  xmlns:exslt-set="http://exslt.org/sets"
  xmlns:exslt-math="http://exslt.org/math"
  xmlns:exslt-functions="http://exslt.org/functions"
  xmlns:exslt-date-time="http://exslt.org/dates-and-times"
  xmlns:dc="&dc;" 
  xmlns:fres="&fres;"
  xmlns:rdf="&rdf;" 
  xmlns:f="&f;" 
  xmlns:vcard="&vcard;" 
  xmlns:fschema="http://schemas.4suite.org/4ss"
  xmlns:cext="http://xmlns.4suite.org/demos/cards"
  extension-element-prefixes="f frdf fhttp fcore exslt exslt-set exslt-math exslt-functions exslt-date-time cext">

  <!--
File:  FILENAME HERE

Description:

    DESCRIPTION HERE


WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information

  -->

  <xsl:template name='award-points'>
    <xsl:param name='player'/>
    <xsl:param name='amount'/>
    <xsl:param name='message'/>

    <AwardPoints player='{$player}' amount='{$amount}'/>

    <xsl:call-template name='log-message'>
      <xsl:with-param name='player' select='$player'/>
      <xsl:with-param name='message' select='$message'/>
    </xsl:call-template>

  </xsl:template>


  <xsl:template name='log-message'>
    <xsl:param name='player'/>
    <xsl:param name='message'/>
    <Log>
      <xsl:if test='$player'>
        <xsl:attribute name='player'>
          <xsl:value-of select='$player'/>
        </xsl:attribute>
      </xsl:if>
      <xsl:value-of select='$message'/>
    </Log>
  </xsl:template>


  <xsl:template name='play-card'>
    <xsl:param name='player'/>
    <xsl:param name='card'/>
    <xsl:param name='played'/>
    <xsl:param name='cur-count'/>

    <!-- first log it -->
    <xsl:call-template name='log-message'>
      <xsl:with-param name='player' select='$player'/>
      <xsl:with-param name='message' select='concat("plays ",$card/@formal-name)'/>
    </xsl:call-template>

    <xsl:comment>
      <xsl:value-of select='key("players",$player)/name'/><xsl:text> plays </xsl:text><xsl:value-of select='$card/@short-name'/>
    </xsl:comment>

    <!-- then move it -->
    <!-- and update the current count -->
    <!-- and change the current player -->
    <Play player='{$player}'><xsl:copy-of select='$card'/></Play>

    <ChangePlayer player='{number($player = 1) + 1}'/>

    <!--Now see if there are any score increases -->
    <xsl:variable name='count-results'>
      <xsl:call-template name='count-stack'>
        <xsl:with-param name='played' select='$played'/>
        <xsl:with-param name='next-card' select='$card'/>
        <xsl:with-param name='cur-count' select='number($cur-count)'/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:if test='$debug="debug"'>

      <xsl:comment>
      Stack Count Results:
      <xsl:for-each select='exslt:node-set($count-results)/StackCountResults/@*'>
        <xsl:value-of select='local-name()'/>=<xsl:value-of select='.'/>,
      </xsl:for-each>
      Played Cards:
      <xsl:for-each select='exslt:node-set($count-results)/StackCountResults/PlayedCards/Card'>
        <xsl:value-of select='@short-name'/>,
      </xsl:for-each>
      Next Card:
      <xsl:value-of select='exslt:node-set($count-results)/StackCountResults/NextCard/Card/@short-name'/> ,
      <xsl:for-each select='exslt:node-set($count-results)/StackCountResults/*'>
        <xsl:value-of select='local-name()'/>,
      </xsl:for-each>
      </xsl:comment>
    </xsl:if>

    <xsl:apply-templates select='exslt:node-set($count-results)/StackCountResults/*' mode='award-points'>
      <xsl:with-param name='player' select='$player'/>
    </xsl:apply-templates>

  </xsl:template>


  <xsl:template name='make-go'>
    <xsl:param name='player'/>
    <xsl:call-template name='log-message'>
      <xsl:with-param name='player' select='$player'/>
      <xsl:with-param name='message' select='"GO"'/>
    </xsl:call-template>

    <xsl:if test='$debug="debug"'>
      <xsl:comment>
        <xsl:value-of select='key("players",$player)/name'/><xsl:text> has a GO!</xsl:text>
      </xsl:comment>
    </xsl:if>
    <SetGo player='{$player}'/>
    <ChangePlayer player='{number($player = 1) + 1}'/>
        
  </xsl:template>



</xsl:stylesheet>
