import os, string, cStringIO

from xml.dom import Node
from Ft.Xml import ReaderException, Domlette, InputSource, FtMiniDom, cDomlette
from Ft.Xml.Lib import TreeCompare
from Ft.Lib import Uri

g_has_pyxml = 0
try:
    from xml.dom import ext
    g_has_pyxml = 1
except ImportError:
    pass

BASE_PATH = os.path.dirname(os.path.abspath(__file__))
BASE_PATH = Uri.OsPathToUri(BASE_PATH) + '/'

def CompareXml(expected, compared):
    try:
        rv = TreeCompare.TreeCompare(expected, compared, baseUri=BASE_PATH)
        return rv
    except Exception, e:
        import traceback
        traceback.print_exc()
        return 1

def can_validate(dom_module):
    # cDomlette doesn't support validation natively
    return g_has_pyxml

def test_reader(tester,domMod):
    tester.startGroup("Dom Readers")
    test_validating_reader(tester, domMod)
    test_nonvalidating_reader(tester, domMod)
    test_xml_base(tester,domMod)
    test_extent_xinclude(tester, domMod)
    test_strip_elements(tester, domMod)
    test_namespaces(tester, domMod)
    test_read_utf16(tester, domMod)
    test_read_exceptions(tester,domMod.nonvalParse)
    test_various_docs(tester, domMod)
    tester.groupDone()
    return

def test_read_utf16(tester,domMod):
    BASE = os.path.join('Xml', 'Core')
    tester.startGroup("Read UTF-16")

    tester.startTest("Good XML: UTF-16LE, LE BOM, utf-16 encoding declaration")
    isrc = InputSource.DefaultFactory.fromUri(os.path.join(BASE, 'goodXml_16LE_LEBOM_16Decl.xml'))
    dom = domMod.nonvalParse(isrc)
    tester.testDone()

    tester.startTest("Good XML: UTF-16BE, BE BOM, utf-16 encoding declaration")
    isrc = InputSource.DefaultFactory.fromUri(os.path.join(BASE, 'goodXml_16BE_BEBOM_16Decl.xml'))
    dom = domMod.nonvalParse(isrc)
    tester.testDone()

    tester.startTest("Good XML: UTF-16LE, LE BOM, no encoding declaration")
    isrc = InputSource.DefaultFactory.fromUri(os.path.join(BASE, 'goodXml_16LE_LEBOM_noDecl.xml'))
    dom = domMod.nonvalParse(isrc)
    tester.testDone()

    tester.startTest("Good XML: UTF-16BE, BE BOM, no encoding declaration")
    isrc = InputSource.DefaultFactory.fromUri(os.path.join(BASE, 'goodXml_16BE_BEBOM_noDecl.xml'))
    dom = domMod.nonvalParse(isrc)
    tester.testDone()

    tester.startTest("Bad XML: UTF-16LE, BE BOM, utf-16 encoding declaration")
    # big-endian BOM will result in little-endian prolog being interpreted as
    # Chinese characters, resulting in a well-formedness error because there
    # is no document element
    uri = os.path.join(BASE, 'badXml_16LE_BEBOM_16Decl.xml')
    isrc = InputSource.DefaultFactory.fromUri(uri)
    tester.testException(domMod.nonvalParse,
                         (isrc,),
                         ReaderException,
                         {'errorCode': ReaderException.XML_PARSE_ERROR}
                         )
    tester.testDone()

    tester.startTest("Bad XML: UTF-16LE, LE BOM, utf-16le encoding declaration")
    # by definition, utf-16le and utf-16be do not have a BOM. if a little-endian
    # BOM is encountered, it is interpreted as a zero-width no-break space, which
    # is not allowed at the beginning of an XML document.
    uri = os.path.join(BASE, 'badXml_16LE_LEBOM_16LEDecl.xml')
    isrc = InputSource.DefaultFactory.fromUri(uri)
##    tester.testException(domMod.nonvalParse,
##                         (isrc,),
##                         ReaderException,
##                         {'errorCode': ReaderException.XML_PARSE_ERROR}
##                         )
    tester.warning("This does not throw an exception")
    tester.testDone()

    tester.startTest("Bad XML: UTF-16LE, no BOM, utf-16 encoding declaration")
    # if the encoding declaration is "utf-16", a BOM is required. without a BOM,
    # UTF-8 will be assumed initially, so the 0x00 bytes will be interpreted as
    # U+0000, which is not allowed in an XML document
    uri = os.path.join(BASE, 'badXml_16LE_noBOM_16Decl.xml')
    isrc = InputSource.DefaultFactory.fromUri(uri)
##    tester.testException(domMod.nonvalParse,
##                         (isrc,),
##                         ReaderException,
##                         {'errorCode': ReaderException.XML_PARSE_ERROR}
##                         )
    tester.warning("This does not throw an exception")
    tester.testDone()

    tester.startTest("Bad XML: UTF-8, no BOM, utf-16 encoding declaration")
    # if the encoding declaration is "utf-16", a BOM is required. without a BOM,
    # UTF-8 will be assumed initially, so the xml declaration is readable, but
    # when the "utf-16" is encountered, a fatal error must occur because there
    # is no BOM. if the parser doesn't catch this error, it should then have a
    # well-formedness error because there is no document element, only a string
    # of non-Latin characters
    uri = os.path.join(BASE, 'badXml_utf8_noBOM_16Decl.xml')
    isrc = InputSource.DefaultFactory.fromUri(uri)
    tester.testException(domMod.nonvalParse,
                         (isrc,),
                         ReaderException,
                         {'errorCode': ReaderException.XML_PARSE_ERROR}
                         )
    tester.testDone()

    tester.groupDone()
    return

def test_strip_elements(tester,domMod):
    tester.startGroup("Strip Elements")

    tester.startTest("Strip Elements")
    src = "<ft:foo xmlns:ft='http://fourthought.com' xmlns:ft2='http://foo.com'><ft:bar>  </ft:bar><ft:bar> F </ft:bar><ft:baz>  Bar1  </ft:baz><ft2:bar>  </ft2:bar><ft2:baz> Bar2 </ft2:baz><bar>  </bar><baz>  Bar3  </baz></ft:foo>"

    stripElements = [(None,'bar',1),
                     ('http://fourthought.com','bar',1),
                     ('http://foo.com','*',1)]

    isrc = InputSource.DefaultFactory.fromString(src, BASE_PATH,
                                                 stripElements=stripElements)
    dom = domMod.nonvalParse(isrc)

    tester.compare(None,dom.documentElement.childNodes[0].firstChild)
    tester.compare(" F ",dom.documentElement.childNodes[1].firstChild.data)
    tester.compare("  Bar1  ",dom.documentElement.childNodes[2].firstChild.data)
    tester.compare(None,dom.documentElement.childNodes[3].firstChild)
    tester.compare(" Bar2 ",dom.documentElement.childNodes[4].firstChild.data)
    tester.compare(None,dom.documentElement.childNodes[5].firstChild)
    tester.compare("  Bar3  ",dom.documentElement.childNodes[6].firstChild.data)
    tester.testDone()

    tester.groupDone()
    return


def test_xml_base(tester, domMod):
    tester.startGroup("XML Base")

    tester.startTest("xml-base")
    src = BASE_TEST

    #strip all ws, set document base URI
    isrc = InputSource.DefaultFactory.fromString(src,"http://rdfInference.org",stripElements=[(None, '*', 1)])
    dom = domMod.nonvalParse(isrc)

    tester.compare("http://rdfInference.org", dom.xmlBase)
    tester.compare("http://rdfInference.org", dom.documentElement.xmlBase)
    tester.compare("http://fourthought.com", dom.documentElement.firstChild.xmlBase)
    #monty
    tester.compare("http://fourthought.com", dom.documentElement.firstChild.firstChild.xmlBase)
    #knight
    tester.compare("http://fourthought.com", dom.documentElement.firstChild.firstChild.firstChild.xmlBase)
    #foo
    tester.compare("http://4suite.org", dom.documentElement.firstChild.childNodes[1].xmlBase)
    #foobar
    tester.compare("http://4suite.org", dom.documentElement.firstChild.childNodes[1].firstChild.xmlBase)

    tester.testDone()

    tester.groupDone()
    return


def test_validating_reader(tester,domMod):
    tester.startGroup("Validating Reader")

    tester.startTest("Read Valid Document")
    if can_validate(domMod):
        source_1 = ADDRBOOK
        isrc = InputSource.DefaultFactory.fromString(source_1, BASE_PATH)
        dom = domMod.valParse(isrc)
        if dom.documentElement:
            tester.compare('ADDRBOOK', dom.documentElement.localName)
        else:
            tester.error("Validating parser returned an empty document")
    else:
        tester.warning("You need PyXML installed to test the validating parser")
    tester.testDone()

    tester.startTest("Read Invalid Document")

    if can_validate(domMod):
        source_1 = ADDRBOOK_INVALID
        isrc = InputSource.DefaultFactory.fromString(source_1, BASE_PATH)
        tester.testException(domMod.valParse,(isrc,),ReaderException)
    else:
        tester.warning("You need PyXML installed to test the validating parser")
    tester.testDone()


    tester.groupDone()
    return


def test_nonvalidating_reader(tester,domMod):
    tester.startGroup("non-Validating Reader")


    tester.startTest("Simple XML")
    src = InputSource.DefaultFactory.fromString(SMALL_XML, BASE_PATH)
    doc = domMod.nonvalParse(src)

    #Test a few of the nodes
    tester.compare(2,len(doc.childNodes))
    tester.compare(Node.PROCESSING_INSTRUCTION_NODE,doc.childNodes[0].nodeType)
    tester.compare("xml-stylesheet",doc.childNodes[0].target)
    tester.compare('href="addr_book1.xsl" type="text/xml"',doc.childNodes[0].data)

    tester.compare(Node.ELEMENT_NODE,doc.childNodes[1].nodeType)
    tester.compare('docelem',doc.childNodes[1].nodeName)
    tester.compare(None,doc.childNodes[1].namespaceURI)
    tester.compare(None,doc.childNodes[1].prefix)
    tester.compare(doc.childNodes[1],doc.documentElement)
    tester.compare(9,len(doc.documentElement.childNodes))

    tester.compare(1,len(doc.documentElement.attributes.keys()))
    tester.compare(('http://www.w3.org/2000/xmlns/', 'ft'),doc.documentElement.attributes.keys()[0])
    tester.compare('http://fourthought.com',doc.documentElement.attributes.values()[0].value)

    tester.compare(1,len(doc.documentElement.childNodes[1].attributes.keys()))
    tester.compare((None,'foo'),doc.documentElement.childNodes[1].attributes.keys()[0])
    tester.compare('bar',doc.documentElement.childNodes[1].attributes.values()[0].value)
    tester.compare('Some Text',doc.documentElement.childNodes[1].childNodes[0].data)


    tester.compare('http://fourthought.com',doc.documentElement.childNodes[5].namespaceURI)
    tester.compare('ft',doc.documentElement.childNodes[5].prefix)
    tester.compare(1,len(doc.documentElement.childNodes[5].attributes.keys()))
    tester.compare(('http://fourthought.com','foo'),doc.documentElement.childNodes[5].attributes.keys()[0])
    tester.compare('nsbar',doc.documentElement.childNodes[5].attributes.values()[0].value)
    tester.testDone()

##    tester.startTest("Parse XML containing undeclared namespace prefix (should be OK)")
##    src = InputSource.DefaultFactory.fromString(QNAME_NO_NSDECL, BASE_PATH)
##    doc = domMod.nonvalParse(src)
##    tester.testDone()
##    tester.groupDone()

    tester.startTest("Parse nasty XML")
    src = InputSource.DefaultFactory.fromString(LARGE_XML, BASE_PATH)
    doc = domMod.nonvalParse(src)
    tester.testDone()

    tester.startTest("Document with general entity defined in external subset (parse method 1)")
    source_1 = GREETING
    isrc = InputSource.DefaultFactory.fromString(source_1, BASE_PATH)
    dom = domMod.nonvalParse(isrc, 1)
    stream = cStringIO.StringIO()
    Domlette.Print(dom, stream)
    expected_1 = """\
<?xml version="1.0" encoding="UTF-8"?>
<greeting>hello world</greeting>
"""
    tester.compare(expected_1, stream.getvalue(), func=TreeCompare.TreeCompare)
    tester.testDone()

    # same as previous test but going through Ft.Xml.Domlette
    tester.startTest("Document with general entity defined in external subset (parse method 2)")
    source_1 = GREETING
    isrc = InputSource.DefaultFactory.fromString(source_1, BASE_PATH)
    from Ft.Xml.Domlette import NonvalidatingReader
    dom = NonvalidatingReader.parse(isrc)
    stream = cStringIO.StringIO()
    Domlette.Print(dom, stream)
    expected_1 = """\
<?xml version="1.0" encoding="UTF-8"?>
<greeting>hello world</greeting>
"""
    tester.compare(expected_1, stream.getvalue(), func=TreeCompare.TreeCompare)
    tester.testDone()
    tester.groupDone()


def test_read_exceptions(tester,parseMethod):
    tester.startGroup("Reader exceptions")

    tester.startTest("Basic Parse Exception")
    tester.warning("Not Tested")
##    tester.testException(reader.fromString,
##                         (BASE_PATH,),
##                         ReaderException,
##                         {'errorCode': ReaderException.XML_PARSE_ERROR}
##                         )
    tester.testDone()


    tester.groupDone()


def test_namespaces(tester,domMod):
    expected = [(None,'xmlns','http://www.w3.org/2000/xmlns/'),
                ('xmlns','are','http://www.w3.org/2000/xmlns/'),
                ('xmlns','foo','http://www.w3.org/2000/xmlns/'),
                ]

    src="""<?xml version="1.0"?>
<diamonds xmlns:are="urn:forever" xmlns:foo="urn:bar" xmlns="http://nowhere"/>"""



    tester.startGroup("Namespaces")

    tester.startTest("Simple test")

    isrc = InputSource.DefaultFactory.fromString(src, BASE_PATH)
    doc = domMod.nonvalParse(isrc)
    attrs = doc.documentElement.attributes.values()
    attrs.sort(lambda a, b: cmp(a.name, b.name))
    tester.compare(3,len(attrs))
    for attr in attrs:
        test = (attr.prefix,attr.localName,attr.namespaceURI)
        tester.compareIn(expected,test)
    tester.testDone()
    tester.groupDone()
    return


def test_extent_xinclude(tester, domMod):
    tester.startGroup("Reader + External Entities + XInclude")

    SRC_1 = """\
<?xml version='1.0'?>
<!DOCTYPE x [ <!ENTITY inc SYSTEM "include1.xml"> ]>
<x>
&inc;
</x>"""
    expected_1="""<?xml version='1.0' encoding='UTF-8'?><x>

<foo/>
</x>"""
    tester.startTest("External entity")

    src = InputSource.DefaultFactory.fromString(SRC_1, BASE_PATH)

    doc = domMod.nonvalParse(src)
    stream = cStringIO.StringIO()
    Domlette.Print(doc, stream)
    tester.compare(expected_1, stream.getvalue(), func=TreeCompare.TreeCompare)
    tester.testDone()

    SRC_2 = """\
<?xml version='1.0'?>
<x xmlns:xi="http://www.w3.org/2001/XInclude">
<xi:include href="include1.xml"/>
</x>"""

    expected_2 = """<?xml version='1.0' encoding='UTF-8'?>
<x xmlns:xi='http://www.w3.org/2001/XInclude'>
<foo/>
</x>"""
    tester.startTest("Basic XInclude")
    src = InputSource.DefaultFactory.fromString(SRC_2, BASE_PATH)
    doc = domMod.nonvalParse(src)
    stream = cStringIO.StringIO()
    Domlette.Print(doc, stream)
    tester.compare(expected_2, stream.getvalue(), func=TreeCompare.NoWsTreeCompare)
    tester.testDone()

    SRC_3="""\
<?xml version='1.0'?>
<x xmlns:xi="http://www.w3.org/2001/XInclude">
<xi:include href="include2.xml"/>
</x>"""
    expected_3="""<?xml version='1.0' encoding='UTF-8'?>
<x xmlns:xi='http://www.w3.org/2001/XInclude'>
<foo>
  <foo/>
</foo>
</x>
"""
    tester.startTest("Recursive XInclude")
    src = InputSource.DefaultFactory.fromString(SRC_3, BASE_PATH)
    doc = domMod.nonvalParse(src)
    stream = cStringIO.StringIO()
    Domlette.Print(doc, stream)
    tester.compare(expected_3, stream.getvalue(), func=TreeCompare.NoWsTreeCompare)
    tester.testDone()

    SRC_4="""\
<?xml version='1.0'?>
<x xmlns:xi="http://www.w3.org/2001/XInclude">
<xi:include href="include2.xml" parse='text'/>
</x>"""
    expected_4="""<?xml version='1.0' encoding='UTF-8'?>
<x xmlns:xi='http://www.w3.org/2001/XInclude'>
&lt;?xml version='1.0' encoding='utf-8'?>
&lt;foo xmlns:xi="http://www.w3.org/2001/XInclude">
  &lt;xi:include href="include1.xml"/>
&lt;/foo>
</x>
"""
    tester.startTest("XInclude text attribute")
    src = InputSource.DefaultFactory.fromString(SRC_4, BASE_PATH)
    doc = domMod.nonvalParse(src)
    stream = cStringIO.StringIO()
    Domlette.Print(doc, stream)
    tester.compare(expected_4, stream.getvalue(),func = TreeCompare.TreeCompare)
    tester.testDone()

    tester.groupDone()
    return

def test_various_docs(tester, domMod):
    tester.startGroup("Simple round trip of various XML (as strings)")
    count = 1
    for doc_text in VARIOUS_STRINGS:
        tester.startTest("Document %d non-validating" % count)
        src = InputSource.DefaultFactory.fromString(doc_text, BASE_PATH)
        doc = domMod.nonvalParse(src)
        stream = cStringIO.StringIO()
        Domlette.Print(doc, stream)
        tester.compare(doc_text, stream.getvalue(), func=CompareXml)
        tester.testDone()

        tester.startTest("Document %d validating" % count)
        if can_validate(domMod):
            #if count == 1 and domMod == FtMiniDom:
            #    tester.warning("xmlproc does not allow ':' in its element names")
            #else:
            if 1:
                src = InputSource.DefaultFactory.fromString(doc_text, BASE_PATH)
                doc = domMod.valParse(src)
                stream = cStringIO.StringIO()
                Domlette.Print(doc, stream)
                tester.compare(doc_text, stream.getvalue(), func=CompareXml)
        else:
            tester.warning("You need PyXML installed to test the validating parser")
        tester.testDone()
    tester.groupDone()

    tester.startGroup("Simple round trip of various XML (as files)")
    count = 1
    for uri in VARIOUS_FILES:
        tester.startTest("Document %d non-validating" % count)
        src = InputSource.DefaultFactory.fromUri(uri, BASE_PATH)
        doc_text = src.read()
        src = InputSource.DefaultFactory.fromUri(uri, BASE_PATH)
        doc = domMod.nonvalParse(src)
        stream = cStringIO.StringIO()
        Domlette.Print(doc, stream)
        tester.compare(doc, stream.getvalue(), func=CompareXml)
        tester.testDone()

        tester.startTest("Document %d validating" % count)
        if can_validate(domMod):
            src = InputSource.DefaultFactory.fromUri(uri, BASE_PATH)
            doc = domMod.valParse(src)
            stream = cStringIO.StringIO()
            Domlette.Print(doc, stream)
            tester.compare(doc_text, stream.getvalue(), func=CompareXml)
        else:
            tester.warning("You need PyXML installed to test the validating parser")
        tester.testDone()

    tester.groupDone()
    return


QNAME_NO_NSDECL = """<?xml version="1.0"?>
<docelem>
  <somePrefix:city>Boulder</somePrefix:city>
</docelem>
"""

SMALL_XML = """<?xml version = "1.0"?>
<?xml-stylesheet href="addr_book1.xsl" type="text/xml"?>
<docelem xmlns:ft='http://fourthought.com'>
  <child foo='bar'>Some Text</child>
  <!--A comment-->
  <ft:nschild ft:foo='nsbar'>Some More Text</ft:nschild>
  <appendChild/>
</docelem>
"""



LARGE_XML="""<?xml version="1.0"?>
<?xml-stylesheet type="text/xsl" href="../../style/function.use-cases.xsl"?>
<!-- <!DOCTYPE exslt:function SYSTEM 'function.dtd'> -->
<exslt:function xmlns:exslt="http://exslt.org/documentation"
                version="1" module="math" status="new">

<exslt:name>min</exslt:name>

<rdf:Description xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
                 xmlns:dc="http://purl.org/dc/elements/1.1/"
                 ID="math:min">
   <dc:subject>EXSLT</dc:subject>
   <dc:subject>math</dc:subject>
   <dc:subject>min</dc:subject>
   <dc:subject>minimum</dc:subject>
   <dc:rights>public domain</dc:rights>
   <exslt:revision>
      <rdf:Description xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
                       xmlns:dc="http://purl.org/dc/elements/1.1/"
                       ID="math:min.1">
         <exslt:version>1</exslt:version>
         <dc:creator email="mail@jenitennison.com"
                     url="http://www.jenitennison.com">Jeni Tennison</dc:creator>
         <dc:date>2001-03-28</dc:date>
         <dc:description>Returns the minimum value from a node-set.</dc:description>
      </rdf:Description>
   </exslt:revision>
</rdf:Description>

<exslt:doc xmlns="">
   <section>
      <para>
         The <function>math:min</function> function returns the minimum, for each node in the argument node-set, of the result of converting the string-values of the node to a number using the <ulink URL="http://www.w3.org/TR/xpath#function-number"> <function>number</function></ulink> function.  The numbers are compared as with the <literal>&lt;</literal> operator.  If the node set is empty, <returnvalue>NaN</returnvalue> is returned.
      </para>
      <para>
         The <literal>math:min</literal> template returns a result tree fragment whose string value is the result of turning the number returned by the function into a string.
      </para>
   </section>
</exslt:doc>

<exslt:definition>
   <exslt:return type="number" />
   <exslt:arg name="nodes" type="node-set" default="/.." />
</exslt:definition>

<exslt:implementations>
   <exslt:implementation src="math.min.function.xsl" language="exslt:exslt"
                         version="1" />
   <exslt:implementation src="math.min.template.xsl" language="exslt:xslt"
                         version="1" />
   <exslt:implementation src="math.min.js" language="javascript"
                         version="1" />
</exslt:implementations>

<exslt:use-cases>
   <exslt:use-case type="example" data="math.min.data.1.xml"
                   xslt="math.min.1.xsl" result="math.min.result.1.xml" />
   <exslt:use-case type="example" template="yes" data="math.min.data.1.xml"
                   xslt="math.min.2.xsl" result="math.min.result.1.xml" />
   <exslt:use-case type="boundary" data="math.min.data.2.xml"
                   xslt="math.min.1.xsl" result="math.min.result.2.xml" />
   <exslt:use-case type="boundary" template="yes" data="math.min.data.2.xml"
                   xslt="math.min.2.xsl" result="math.min.result.2.xml" />
   <exslt:use-case type="error" data="math.min.data.1.xml"
                   xslt="math.min.3.xsl">
      <exslt:doc xmlns="">
         <para>
            This use case shows an error when the function is passed a
            number as the value of the first argument.
         </para>
      </exslt:doc>
   </exslt:use-case>
   <exslt:use-case type="error" template="yes" data="math.min.data.1.xml"
                   xslt="math.min.4.xsl">
      <exslt:doc>
         <para>
            This use case shows an error when the function is passed a
            number as the value of the <parameter>nodes</parameter>
            parameter.
         </para>
      </exslt:doc>
   </exslt:use-case>
</exslt:use-cases>

</exslt:function>"""


GREETING = """<?xml version = "1.0"?>
<!DOCTYPE greeting SYSTEM "greeting.dtd">
<greeting>&bar;</greeting>
"""


ADDRBOOK = """<?xml version = "1.0"?>
<!DOCTYPE ADDRBOOK SYSTEM "addr_book.dtd">
<ADDRBOOK>
    <ENTRY ID="pa">
        <NAME>Pieter Aaron</NAME>
        <ADDRESS>404 Error Way</ADDRESS>
        <PHONENUM DESC="Work">404-555-1234</PHONENUM>
        <PHONENUM DESC="Fax">404-555-4321</PHONENUM>
        <PHONENUM DESC="Pager">404-555-5555</PHONENUM>
        <EMAIL>pieter.aaron@inter.net</EMAIL>
    </ENTRY>
    <ENTRY ID="en">
        <NAME>Emeka Ndubuisi</NAME>
        <ADDRESS>42 Spam Blvd</ADDRESS>
        <PHONENUM DESC="Work">767-555-7676</PHONENUM>
        <PHONENUM DESC="Fax">767-555-7642</PHONENUM>
        <PHONENUM DESC="Pager">800-SKY-PAGEx767676</PHONENUM>
        <EMAIL>endubuisi@spamtron.com</EMAIL>
    </ENTRY>
    <ENTRY ID="vz">
        <NAME>Vasia Zhugenev</NAME>
        <ADDRESS>2000 Disaster Plaza</ADDRESS>
        <PHONENUM DESC="Work">000-987-6543</PHONENUM>
        <PHONENUM DESC="Cell">000-000-0000</PHONENUM>
        <EMAIL>vxz@magog.ru</EMAIL>
    </ENTRY>
</ADDRBOOK>"""


ADDRBOOK_INVALID = """<?xml version = "1.0"?>
<!DOCTYPE ADDRBOOK SYSTEM "addr_book.dtd">
<ADDRBOOK>
    <INVALIDENTRY ID="pa">
        <NAME>Pieter Aaron</NAME>
        <ADDRESS>404 Error Way</ADDRESS>
        <PHONENUM DESC="Work">404-555-1234</PHONENUM>
        <PHONENUM DESC="Fax">404-555-4321</PHONENUM>
        <PHONENUM DESC="Pager">404-555-5555</PHONENUM>
        <EMAIL>pieter.aaron@inter.net</EMAIL>
    </INVALIDENTRY>
</ADDRBOOK>"""


BASE_TEST = """<?xml version = "1.0"?>
<spam>
  <eggs xml:base="http://fourthought.com">
    <monty python="nicht">knight</monty>
    <foo xml:base="http://4suite.org" bar="baz">foobar</foo>
  </eggs>
</spam>
"""

VARIOUS_STRINGS = [
#XML well-formed but not XMLNS well-formed
"""<?xml version="1.0"?>
<!DOCTYPE foo SYSTEM "various_strings_1.dtd">
<foo undeclared-prefix:bar=''><undeclared-prefix:bar/></foo>""",
]

#Place stuff that cannot be in Python files as separate files listed here
VARIOUS_FILES = [
]

