from Ft.Lib import TestSuite
from Ft.Xml import XUpdate
from Ft.Xml import Domlette, InputSource
from Ft.Xml.Lib import TreeCompare
import cStringIO

#From the spec

src_1 = """\
<?xml version="1.0"?>
<addresses version="1.0">

  <address id="1">
    <fullname>Andreas Laux</fullname>
    <born day='1' month='12' year='1978'/>
    <town>Leipzig</town>
    <country>Germany</country>
  </address>

</addresses>
"""

xu_1 = """\
<?xml version="1.0"?>
<xupdate:modifications
  version="1.0"
  xmlns:xupdate="http://www.xmldb.org/xupdate"
>

  <xupdate:insert-after select="/addresses/address[1]" >

    <xupdate:element name="address">
      <xupdate:attribute name="id">2</xupdate:attribute>
      <fullname>Lars Martin</fullname>
      <born day='2' month='12' year='1974'/>
      <town>Leizig</town>
      <country>Germany</country>
    </xupdate:element>
  </xupdate:insert-after>

</xupdate:modifications>
"""

expected_1 = """<?xml version='1.0' encoding='UTF-8'?>
<addresses version='1.0'>

  <address id='1'>
    <fullname>Andreas Laux</fullname>
    <born day='1' month='12' year='1978'/>
    <town>Leipzig</town>
    <country>Germany</country>
  </address><address id='2'><fullname>Lars Martin</fullname><born day='2' month='12' year='1974'/><town>Leizig</town><country>Germany</country></address>

</addresses>"""

src_2 = """\
<?xml version="1.0"?>
<addresses>
  <address>
    <town>Los Angeles</town>
  </address>
</addresses>
"""

xu_2 = """\
<?xml version="1.0"?>
<xupdate:modifications
  version="1.0"
  xmlns:xupdate="http://www.xmldb.org/xupdate"
>

  <xupdate:append select="/addresses" child="last()">
    <xupdate:element name="address">
      <town>San Francisco</town>
    </xupdate:element>
  </xupdate:append>

</xupdate:modifications>
"""

expected_2 = """<?xml version='1.0' encoding='UTF-8'?>
<addresses>
  <address>
    <town>Los Angeles</town>
  </address>
<address><town>San Francisco</town></address></addresses>"""

src_3 = """\
<?xml version="1.0"?>
<addresses>
  <address>
    <town>Los Angeles</town>
  </address>
  <address>
    <town>San Francisco</town>
  </address>
</addresses>
"""

xu_3 = """\
<?xml version="1.0"?>
<xupdate:modifications
  version="1.0"
  xmlns:xupdate="http://www.xmldb.org/xupdate"
>

  <xupdate:update select="/addresses/address[2]/town">
    New York
  </xupdate:update>

</xupdate:modifications>
"""

expected_3 = """<?xml version='1.0' encoding='UTF-8'?>
<addresses>
  <address>
    <town>Los Angeles</town>
  </address>
  <address>
    <town>
    New York
  </town>
  </address>
</addresses>"""

src_4 = src_3

xu_4 = """\
<?xml version="1.0"?>
<xupdate:modifications
  version="1.0"
  xmlns:xupdate="http://www.xmldb.org/xupdate"
>

  <xupdate:remove select="/addresses/address[1]"/>

</xupdate:modifications>
"""

expected_4 = """<?xml version='1.0' encoding='UTF-8'?>
<addresses>
  
  <address>
    <town>San Francisco</town>
  </address>
</addresses>"""

src_5 = """<ftss:Container xmlns:ftss="http://xmlns.4suite.org/reserved">
  <ftss:Children/>
</ftss:Container>"""


xu_5 = """\
<?xml version="1.0"?>
<xupdate:modifications
  version="1.0"
  xmlns:xupdate="http://www.xmldb.org/xupdate"
  xmlns:ftss="http://xmlns.4suite.org/reserved"
  xmlns:xlink="http://www.w3.org/1999/xlink"
>
<xupdate:append select="(ftss:Repository | ftss:Container)/ftss:Children" child="last()">
    <ftss:ChildReference xlink:type="simple" xlink:actuate="onLoad" xlink:show="embed">
      <xupdate:attribute name='xlink:href'><xupdate:value-of select='concat($child-name,";metadata")'/></xupdate:attribute>
    </ftss:ChildReference>
  </xupdate:append>
</xupdate:modifications>
"""

expected_5 = """<?xml version='1.0' encoding='UTF-8'?>
<ftss:Container xmlns:xlink='http://www.w3.org/1999/xlink' xmlns:ftss='http://xmlns.4suite.org/reserved'>\n  <ftss:Children><ftss:ChildReference xlink:href='FOO;metadata' xlink:type='simple' xlink:actuate='onLoad' xlink:show='embed'/></ftss:Children>\n</ftss:Container>"""


#etc

xureader = XUpdate.Reader()
from Ft.Xml import cDomlette, FtMiniDom

def Test(tester):
    tester.startGroup("XUpdate")
    for reader,title in [(FtMiniDom.nonvalParse,"minidom"),
                         (cDomlette.nonvalParse,"cDomlette"),]:
        tester.startGroup("Source from " + title)
        test_reader(tester,reader)
        tester.groupDone()
    tester.groupDone()

def test_reader(tester,srcReader):
    processor = XUpdate.Processor()

    # ------------------------------------------------------

    for name,src,xuSrc,expected,vars in [("XUpdate spec example 1",src_1,xu_1,expected_1,{}),
                                         ("Append",src_2,xu_2,expected_2,{}),
                                         ("Update",src_3,xu_3,expected_3,{}),
                                         ("Remove",src_4,xu_4,expected_4,{}),
                                         ("Attribute/Value-Of",src_5,xu_5,expected_5,{(None,'child-name'):"FOO"}),
                                         ]:

        tester.startTest(name)
        isrc = InputSource.DefaultFactory.fromString(src,'xu')
        src = srcReader(isrc)
        isrc = InputSource.DefaultFactory.fromString(xuSrc,'xu')
        xu = xureader.fromSrc(isrc)

        processor.execute(src, xu,variables = vars)
        st = cStringIO.StringIO()
        Domlette.Print(src, stream=st)
    
        tester.compare(expected, st.getvalue(),
                       func=TreeCompare.TreeCompare, diff=1)

        tester.testDone()

    return
    

    tester.groupDone()
    return
