########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Xml/Lib/StreamWriter.py,v 1.3 2003/03/15 00:37:47 mbrown Exp $
"""
Python version of an encoded character stream writer that makes
substititions of illegal and unencodable characters

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

import codecs, re

ILLEGAL_XML_CHAR_PATTERN = re.compile(u'[^\x09\x0A\x0D\x20-\uD7FF\uE000-\uFFFD\u10000-\u10FFFF]')

#HTML's SGML declaration prohibits U+0080 through U+009F
#ILLEGAL_HTML_CHAR_PATTERN = re.compile(u'[^\x09\x0A\x0D\x20-\x7F\xA0-\uD7FF\uE000-\uFFFD\u10000-\u10FFFF]')

_ASCII = u''.join(map(unichr, range(128)))

class StreamWriter:

    def __init__(self, stream, encoding):
        """
        Creates a StreamWriter instance.

        stream must be a file-like object open for writing (binary) data.
        encoding specifies the encoding which is to be used for the stream.
        """
        self.stream = stream
        self.encoding = encoding
        self._rawwrite = stream.write
        self._encode = codecs.lookup(encoding)[0]

        # Shortcut ASCII writing if the codec encodes it as single bytes
        if len(self._encode(_ASCII)[0]) == 128:
            self.writeAscii = stream.write
        else:
            self.writeAscii = self.writeEncode
        return

    def __str__(self):
        stream = getattr(self.stream, 'name', repr(self.stream))
        return '<%s, stream=%r, encoding=%r>' % (self.__class__.__name__,
                                                 stream, self.encoding)
    
    def writeEncode(self, string, where='output'):
        """
        Writes the string encoded to self.stream.

        Raises ValueError if string cannot be encoded.
        """
        try:
            data, consumed = self._encode(string, 'strict')
        except:
            raise ValueError('Invalid character in %s %r' % (where, string))
        else:
            self._rawwrite(data)
        return
            
    def writeEscape(self, string, entityMap):
        """
        Writes the string encoded to self.stream.

        If string cannot be encoded, the bad characters are replaced
        with their numerical character entity.  Illegal characters are
        replaced with a '?'.
        """
        # Replace any illegal characters with '?'.
        string = ILLEGAL_XML_CHAR_PATTERN.sub(u'?', string)

        # Replace characters with their character entities
        string = entityMap._replace(lambda m, d=entityMap.entities:
                                    d[m.group()], string)

        # Replace any characters not representable in this encoding with
        # their numerical character entity.
        try:
            data, consumed = self._encode(string, 'strict')
        except:
            for char in string:
                try:
                    data = self._encode(char, 'strict')[0]
                except:
                    # Note: use decimal form due to some broken browsers.
                    self.writeAscii('&#%d;' % ord(char))
                else:
                    self._rawwrite(data)
        else:
            self._rawwrite(data)
        return

class EntityMap:

    def __init__(self, mapping):
        self.entities = mapping
        self._regexp = re.compile('['+''.join(mapping.keys())+']')
        self._replace = self._regexp.sub
        return

try:
    from cStreamWriter import *
except ImportError:
    pass
