########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Xml/Xslt/GenericVariable.py,v 1.16 2003/07/01 23:34:52 jkloth Exp $
"""
Base implementation of XSLT variable assigning elements

Copyright 2002 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

from Ft.Xml import EMPTY_NAMESPACE
from Ft.Xml.Xslt import XsltElement, XsltException, Error, XSL_NAMESPACE
from Ft.Xml.Xslt import ContentInfo, AttributeInfo
from Ft.Xml.XPath import FT_EXT_NAMESPACE

# for variable binding
from Ft.Xml.Xslt.StylesheetTree import XsltNode

# for variable shadowing checks
from Ft.Xml.Xslt.TemplateElement import TemplateElement

class GenericVariableElement(XsltElement):

    content = ContentInfo.Template
    legalAttrs = {
        'name' : AttributeInfo.QName(required=1),
        'select' : AttributeInfo.Expression(),
        'f:node-set' : AttributeInfo.YesNoAvt(default='no'),
        }

    doesSetup = 1

    def setup(self):
        # check for a bad binding
        if self._select and self.children:
            raise XsltException(Error.VAR_WITH_CONTENT_AND_SELECT, self._name)

        # See the bottom of this file for these helper "nodes"
        binding_save = self.parent.children[0]
        if not isinstance(binding_save, PushVariablesNode):
            # varBindings not yet saved for this level in the stylesheet tree
            parent = self.parent

            binding_save = PushVariablesNode(parent.root, parent.baseUri)
            parent.insertChild(0, binding_save)
            parent.root.primeInstructions.append(binding_save)
        return binding_save

    def instantiate(self, context, processor):
        # NOTE: all we want to do is change the varBindings
        context.processorNss = self.namespaces
        context.currentInstruction = self

        if self._select:
            result = self._select.evaluate(context)
        elif self.children:
            processor.pushResultTree(self.baseUri)
            context = reduce(lambda context, child, processor=processor:
                             child.instantiate(context, processor)[0],
                             self.children, context)
            result = processor.popResult()
            # Why is the check for childNodes necessary?  This will always
            # be an RTF.
            if self.attributes.get((FT_EXT_NAMESPACE, 'node-set')) == 'yes' \
                   and hasattr(result, 'childNodes'):
                result = [result]
        else:
            result = u""
        context.varBindings[self._name] = result

        return (context,)

# pseudo-nodes for save/restore of variable bindings
class PushVariablesNode(XsltNode):

    def __init__(self, root, baseUri):
        XsltNode.__init__(self, root, baseUri)
        self.savedVariables = []
        self.popNode = PopVariablesNode(self.savedVariables)
        self._is_primed = 0
        return

    def prime(self, processor, context):
        if not self._is_primed:
            self.parent.children.append(self.popNode)
            self._is_primed = 1
        return

    def instantiate(self, context, processor):
        self.savedVariables.append(context.varBindings)
        context.varBindings = context.varBindings.copy()
        return (context,)

    def isPseudoNode(self):
        return 1


class PopVariablesNode(XsltNode):

    def __init__(self, savedVariables):
        self.savedVariables = savedVariables
        return

    def instantiate(self, context, processor):
        context.varBindings = self.savedVariables.pop()
        return (context,)

    def isPseudoNode(self):
        return 1
