import sys
from Ft.Lib.DistExt import Localization, Script, Extension, Document

# -- Prepare expat for cDomlette (inspired by PyXML) -------------------

# We cannot coexist with a staticly linked expat (symbol clashes).
# Exit if Python was compilied with a built-in expat.
if 'pyexpat' in sys.builtin_module_names:
    raise SystemExit("""
Error: your Python installation has expat built-in statically, which will
cause conflicts with the expat 4Suite provides.  Please re-build Python so
that pyexpat.so is created as a dynamic module, only.  This is the default
if you build from source on most platforms.
""")

# To build expat 1.95.6, we need to find out the byteorder
if hasattr(sys, 'byteorder') and sys.byteorder != "little":
    byteorder = "4321"
else:
    byteorder = "1234"
del sys

# -- package definition ------------------------------------------------

name = "4Xml"
version = '1.0a3'
description = 'XML processing libraries'
author = "Fourthought, Inc."
author_email = "support@4suite.org"
url = "http://4suite.org/"
long_description = "XML tools and libraries for Python: Domlette, XPath, XSLT, XPointer, XLink, XUpdate"

validate_templates = ['exclude Ft/Xml/Lib/*.po',
                      ]

manifest_templates = ['include Ft/Xml/src/expat/COPYING',
                      ]

packages = ['Ft.Xml',
            'Ft.Xml.Lib',
            'Ft.Xml.FtMiniDom',
            'Ft.Xml.XPath',
            'Ft.Xml.XPointer',
            'Ft.Xml.Xslt',
            'Ft.Xml.Xslt.Exslt',
            'Ft.Xml.Xslt.Debugger',
            'Ft.Xml.XLink',
            ]

tests = ['Xml',
         'Xml.Core',
         'Xml.Borrowed',
         'Xml.XPath',
         'Xml.XPath.Core',
         'Xml.XPath.Borrowed',
         'Xml.XPointer',
         'Xml.Xslt',
         'Xml.Xslt.Core',
         'Xml.Xslt.Core.etc',
         'Xml.Xslt.Borrowed',
         'Xml.Xslt.Borrowed.etc',
         'Xml.XLink',
         ]

data_files = [
    #Install the profile tests
    ('Ft.Share.Profile.Xml.XUpdate',['profile/Xml/XUpdate/*.*']),
    #Install the default catalog
    ('Ft.Share',['Ft/Xml/*.cat']),
    ]

ext_modules = [Extension('Ft.Xml.Lib.routines',
                         include_dirs=['Ft/Xml/src','Ft/Lib/src'],
                         sources=['Ft/Xml/src/routines.c'],
                         ),
               Extension('Ft.Xml.Lib.cStreamWriter',
                         sources=['Ft/Xml/src/StreamWriter.c'],
                         ),
               Extension('Ft.Xml.XPath.XPathParserc',
                         sources=['Ft/Xml/XPath/XPathParser.c'],
                         ),
               Extension('Ft.Xml.XPath._conversions',
                         include_dirs=['Ft/Xml/src', 'Ft/Lib/src'],
                         sources=['Ft/Xml/XPath/_conversions.c'],
                         ),
               Extension('Ft.Xml.SplitQNamec',
                         define_macros=[("SPLITQNAME","StandAloneSplitQName")],
                         include_dirs=['Ft/Xml/src/domlette'],
                         sources=['Ft/Xml/src/splitqname.c',
                                  'Ft/Xml/src/domlette/splitqname_func.c',
                                  ],
                         ),
               Extension('Ft.Xml.Lib.XmlStripc',
                         include_dirs=['Ft/Xml/src/domlette'],
                         sources=['Ft/Xml/src/xmlstrip.c'],
                         ),
               Extension('Ft.Xml.cDomlettec',
                         define_macros=[('BYTEORDER', byteorder),
                                        ('XML_UNICODE', None),
                                        ],
                         include_dirs=['Ft/Xml/src',
                                       'Ft/Xml/src/expat/lib',
                                       ],
                         sources=['Ft/Xml/src/expat/lib/xmlparse.c',
                                  'Ft/Xml/src/expat/lib/xmlrole.c',
                                  'Ft/Xml/src/expat/lib/xmltok.c',

                                  'Ft/Xml/src/domlette/domlette.c',
                                  'Ft/Xml/src/domlette/attr.c',
                                  'Ft/Xml/src/domlette/element.c',
                                  'Ft/Xml/src/domlette/processinginstruction.c',
                                  'Ft/Xml/src/domlette/characterdata.c',
                                  'Ft/Xml/src/domlette/comment.c',
                                  'Ft/Xml/src/domlette/text.c',
                                  'Ft/Xml/src/domlette/document.c',
                                  'Ft/Xml/src/domlette/documentfragment.c',
                                  'Ft/Xml/src/domlette/domimplementation.c',
                                  'Ft/Xml/src/domlette/node.c',
                                  'Ft/Xml/src/domlette/util.c',
                                  'Ft/Xml/src/domlette/splitqname_func.c',
                                  'Ft/Xml/src/domlette/state_machine.c',
                                  'Ft/Xml/src/domlette/parse_event_handler.c',
                                  'Ft/Xml/src/domlette/expat_module.c',
                                  'Ft/Xml/src/domlette/reader.c',
                                  'Ft/Xml/src/domlette/ds_tools.c',
                                  'Ft/Xml/src/domlette/ds_primitives.c',
                                  'Ft/Xml/src/domlette/getallns.c',
                                  'Ft/Xml/src/domlette/xptr.c',
                                  ],
                         ),
               Extension('Ft.Xml.XPointer.XPtrExprParserc',
                         sources=['Ft/Xml/XPointer/XPtrExprParser.c'],
                         ),

               Extension('Ft.Xml.XPointer.XPointerParserc',
                         sources=['Ft/Xml/XPointer/XPointerParser.c'],
                         ),

               Extension('Ft.Xml.Xslt.XPatternParserc',
                         sources=['Ft/Xml/Xslt/XPatternParser.c'],
                         ),
               ]

l10n = [Localization(package='Ft.Xml.XPath',
                     domain='4Suite',
                     sources=['Ft/Xml/XPath/MessageSource.py']),
        ]

scripts = [Script('4xslt',
                  'Ft.Xml.Xslt._4xslt.XsltCommandLineApp'),
           Script('4xpath',
                  'Ft.Xml.XPath._4xpath.XPathCommandLineApp'),
           Script('4xupdate',
                  'Ft.Xml.Lib._4xupdate.XUpdateCommandLineApp'),
           Script('4xml',
                  'Ft.Xml._4xml.XmlCommandLineApp'),
           ]

bgen_files = ['Ft/Xml/XPath/XPath.bgen',
              'Ft/Xml/XPointer/XPointer.bgen',
              'Ft/Xml/XPointer/XPtrExpr.bgen',
              'Ft/Xml/Xslt/XPattern.bgen',
              ]

doc_files = [Document('docs/xml/exslt.doc',
                      '',
                      htmlStylesheets=['Ft/Data/docbook_html1.xslt'],
                      htmlOutfile='exslt.html',
                      textStylesheets=['Ft/Data/docbook_text1.xslt'],
                      textOutfile='exslt.txt',
                      title='EXSLT for 4Suite'
                      ),
             Document('docs/xml/HowTos/cjkv.doc',
                      'HowTos',
                      htmlStylesheets=['Ft/Data/docbook_html1.xslt'],
                      htmlOutfile='cjkv.html',
                      #textStylesheets=['Ft/Data/docbook_text1.xslt'],
                      #textOutfile='cjkv.txt',
                      title='Chinese, Japanese, Korean, and Vietnamese encodings with 4Suite'
                      ),
             ]

dependencies = [('4Lib','1.0a3'),
                ]

# Cleanup local namespace
del byteorder
del Document
del Localization
del Script
del Extension

